@@ -1,18 +1,51 @@
 """The "Actions" widget"""
 
-import cola
+from PyQt4 import QtCore
+from PyQt4 import QtGui
+
 from cola import cmds
-from cola import qt
-from cola import stash
 from cola.i18n import N_
+from cola.models.selection import selection_model
+from cola.widgets import defs
 from cola.widgets import remote
-from cola.qt import create_button
+from cola.widgets import stash
+from cola.qtutils import create_button
 from cola.qtutils import connect_button
 
 
-class ActionButtons(qt.QFlowLayoutWidget):
+class QFlowLayoutWidget(QtGui.QWidget):
+
+    _horizontal = QtGui.QBoxLayout.LeftToRight
+    _vertical = QtGui.QBoxLayout.TopToBottom
+
+    def __init__(self, parent):
+        QtGui.QWidget.__init__(self, parent)
+        self._direction = self._vertical
+        self._layout = layout = QtGui.QBoxLayout(self._direction)
+        layout.setSpacing(defs.spacing)
+        layout.setMargin(defs.margin)
+        self.setLayout(layout)
+        policy = QtGui.QSizePolicy(QtGui.QSizePolicy.Minimum,
+                                   QtGui.QSizePolicy.Minimum)
+        self.setSizePolicy(policy)
+        self.setMinimumSize(QtCore.QSize(1, 1))
+        self.aspect_ratio = 0.8
+
+    def resizeEvent(self, event):
+        size = event.size()
+        if size.width() * self.aspect_ratio < size.height():
+            dxn = self._vertical
+        else:
+            dxn = self._horizontal
+
+        if dxn != self._direction:
+            self._direction = dxn
+            self.layout().setDirection(dxn)
+
+
+class ActionButtons(QFlowLayoutWidget):
     def __init__(self, parent=None):
-        qt.QFlowLayoutWidget.__init__(self, parent)
+        QFlowLayoutWidget.__init__(self, parent)
         layout = self.layout()
         self.stage_button = create_button(text=N_('Stage'), layout=layout)
         self.unstage_button = create_button(text=N_('Unstage'), layout=layout)
@@ -36,7 +69,7 @@ class ActionButtons(qt.QFlowLayoutWidget):
 
     def stage(self):
         """Stage selected files, or all files if no selection exists."""
-        paths = cola.selection_model().unstaged
+        paths = selection_model().unstaged
         if not paths:
             cmds.do(cmds.StageModified)
         else:
@@ -44,7 +77,7 @@ class ActionButtons(qt.QFlowLayoutWidget):
 
     def unstage(self):
         """Unstage selected files, or all files if no selection exists."""
-        paths = cola.selection_model().staged
+        paths = selection_model().staged
         if not paths:
             cmds.do(cmds.UnstageAll)
         else:
